#! /usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs = require("yargs");
var randomWords = require("random-words");
const nanospinner_1 = require("nanospinner");
const upload_1 = require("./commands/upload");
const login_1 = require("./commands/login");
const create_configuration_1 = require("./commands/create-configuration");
const create_organization_1 = require("./commands/create-organization");
const init_1 = require("./commands/init");
const publish_1 = require("./commands/publish");
const prompts_1 = require("./prompts/prompts");
const configuration_1 = __importDefault(require("./configuration"));
const utils_1 = require("./utils");
(() => __awaiter(void 0, void 0, void 0, function* () {
    console.log(`Spheron CLI ${configuration_1.default.version}`);
    const options = yargs
        .usage("Usage: $0 init, create-app, login, upload, publish, create-organization")
        .command("login", "Login to the system", (yargs) => {
        yargs
            .option("github", {
            describe: "Login using Github credentials",
            type: "boolean",
        })
            .option("gitlab", {
            describe: "Login using Gitlab credentials",
            type: "boolean",
        })
            .option("bitbucket", {
            describe: "Login using Bitbucket credentials",
            type: "boolean",
        });
    })
        .command("upload", "Upload", (yargs) => {
        yargs
            .option("path", {
            describe: "Path to file",
            type: "string",
            demandOption: false,
        })
            .option("protocol", {
            describe: "Upload protocol",
            choices: ["arweave", "ipfs-filecoin", "ipfs"],
        })
            .option("projectname", {
            describe: "Project name",
            type: "string",
        })
            .option("organization", {
            describe: "Organization where project will be created",
            type: "string",
        });
    })
        .command("publish", "Upload your project setup in spheron.json")
        .command("create-configuration", "Create spheron config file")
        .command("create-organization", "Create organization", (yargs) => {
        yargs
            .option("name", {
            describe: "Name of the organization",
            type: "string",
        })
            .option("username", {
            describe: "Username of the organization",
            type: "string",
        });
    })
        .command("init", "Spheron file initialization in project", (yargs) => {
        yargs.option("protocol", {
            describe: "Protocol that will be used for uploading ",
            type: "string",
            demandOption: false,
        });
        yargs.option("name", {
            describe: "Project name",
            type: "string",
            demandOption: false,
        });
        yargs.option("path", {
            describe: "Path to uploading content",
            type: "string",
            demandOption: false,
        });
    })
        .command("create-app", "Create a template application which can run on Spheron out of the box").argv;
    if (options._[0] === "login") {
        const validOptions = ["github", "gitlab", "bitbucket"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (() => __awaiter(void 0, void 0, void 0, function* () {
            if (options.github) {
                yield (0, login_1.login)("github");
            }
            else if (options.gitlab) {
                yield (0, login_1.login)("gitlab");
            }
            else if (options.bitbucket) {
                yield (0, login_1.login)("bitbucket");
            }
            else {
                const { provider } = yield (0, prompts_1.promptForLogin)();
                yield (0, login_1.login)(provider);
            }
        }))();
    }
    if (options._[0] === "upload") {
        const validOptions = ["path", "protocol", "projectname", "organization"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (() => __awaiter(void 0, void 0, void 0, function* () {
            let path, protocol, organizationId, projectName;
            if (options.path && options.protocol) {
                path = options.path;
                protocol = options.protocol;
                organizationId = options.organization;
                projectName = options.projectname;
            }
            else {
                let prompt = yield (0, prompts_1.promptForUploadFile)();
                path = prompt.path;
                protocol = prompt.protocol;
                organizationId = prompt.organizationId;
                projectName = prompt.projectName;
            }
            if (!projectName) {
                projectName = `${randomWords()}-${randomWords()}`;
                console.log(`Generated default project name: ${projectName}`);
            }
            if (!organizationId) {
                organizationId = yield (0, utils_1.readFromJsonFile)("organization", configuration_1.default.configFilePath);
            }
            if (!path) {
                path = "./";
            }
            const fileType = yield (0, utils_1.getFileType)(path);
            const spinner = (0, nanospinner_1.createSpinner)(`Uploading ${fileType} `).start();
            try {
                yield (0, upload_1.upload)(path, protocol, organizationId, projectName);
            }
            catch (error) {
                spinner.error();
                process.exit(1);
            }
            spinner.success();
        }))();
    }
    if (options._[0] === "publish") {
        (() => __awaiter(void 0, void 0, void 0, function* () {
            const spinner = (0, nanospinner_1.createSpinner)(`Publish `).start();
            try {
                yield (0, publish_1.publish)();
            }
            catch (error) {
                spinner.error();
                process.exit(1);
            }
            spinner.success();
        }))();
    }
    if (options._[0] === "create-configuration") {
        (() => __awaiter(void 0, void 0, void 0, function* () {
            (0, create_configuration_1.createConfiguration)();
        }))();
    }
    if (options._[0] === "create-organization") {
        const validOptions = ["name", "username"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (() => __awaiter(void 0, void 0, void 0, function* () {
            try {
                let name, username;
                if (options.name && options.username) {
                    name = options.name;
                    username = options.username;
                }
                else {
                    const prompt = yield (0, prompts_1.promptForCreateOrganization)();
                    name = prompt.name;
                    username = prompt.username;
                }
                if (!name) {
                    throw new Error("Please insert a name for organization.");
                }
                if (!username) {
                    throw new Error("Please insert username for organization.");
                }
                const spinner = (0, nanospinner_1.createSpinner)(`Creating organization name: ${name}, username: ${username} `).start();
                console.log("");
                try {
                    yield (0, create_organization_1.createOrganization)(name, username, "app");
                }
                catch (error) {
                    spinner.error();
                    process.exit(1);
                }
                spinner.success();
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        }))();
    }
    if (options._[0] === "init") {
        const validOptions = ["protocol", "name", "path"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (() => __awaiter(void 0, void 0, void 0, function* () {
            try {
                let name, protocol, path;
                if (options.protocol) {
                    name = options.name;
                    protocol = options.protocol;
                    path = options.path;
                }
                else {
                    const prompt = yield (0, prompts_1.promptForInit)();
                    name = prompt.name;
                    protocol = prompt.protocol;
                    path = prompt.path;
                }
                if (!name) {
                    name = `${randomWords()}-${randomWords()}`;
                }
                if (!path) {
                    path = "./";
                }
                yield (0, init_1.init)(name, protocol, path);
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        }))();
    }
    if (options._[0] === "create-app") {
        (() => __awaiter(void 0, void 0, void 0, function* () {
            try {
                yield (0, prompts_1.promptForCreateApp)();
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        }))();
    }
    if (!options._[0]) {
        console.log("Please use --help to check all commands available with spheron cli");
    }
}))();
